package applicationLogic;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import storagePackage.storageDataAccess;

/**
 * The Class Manager.
 */
public class Manager {
	
	/** The print list. */
	private ArrayList<Printing> printList;
	
	/** The instance. */
	private static Manager instance = null;
	
	/**
	 * Instantiates a new manager.
	 */
	private Manager()
	{
		printList = new ArrayList<Printing>();
	}
	
	/**
	 * Adds the printing.
	 *
	 * @param toAdd the to add
	 * @return the int
	 */
	public int addPrinting(Printing toAdd)
	{
		printList.add(toAdd);
		return toAdd.getId();
	}
	
	/**
	 * Delete printing.
	 *
	 * @param id the id
	 * @return true, if successful
	 */
	public boolean deletePrinting(int id) throws BadPrintingException
	{
		Iterator<Printing> it = printList.iterator();
			while (it.hasNext())
				{
				Printing actual = it.next();
				if (actual.getId() == id)
				{
					//look if printing is Zeitschrift and search for referenced articles
					if (actual instanceof Magazine)
					{
						Iterator<Printing> delIterator = printList.iterator();
						while (delIterator.hasNext()) {
							Printing testForZeitschriftenRef = delIterator.next();
							if (testForZeitschriftenRef instanceof Article) {
								Article toTest = (Article) testForZeitschriftenRef;
								if (toTest.getMagazine() == actual) return false;
							}
						}
					}
					it.remove();
					return true;
				}
			}
			//Object wasn't found in list
			throw new BadPrintingException(id);
	}
	
	/**
	 * filter printings by type. String filterType must be classname from object to filter
	 *
	 * @param filterType the filter type
	 * @return ArrayList from filtered printings
	 */
	public ArrayList<Printing> filterPrintings(String filterType)
	{
		ArrayList<Printing> result = new ArrayList<Printing>();
		for (Printing actual : printList)
		{
			//Always compare strings by equals not by ==
			if (actual.getClass().getSimpleName().equals(filterType))
			{
				result.add(actual);
			}
		}
		return result;
	}
	
	/**
	 * Search printings.
	 *
	 * @param title the title
	 * @return the array list
	 */
	public ArrayList<Printing> searchPrintings(String title)
	{
		ArrayList<Printing> result = new ArrayList<Printing>();
		for (Printing actual : printList)
		{
			if (actual.getTitle().equals(title))
			{
				result.add(actual);
			}
		}
		return result;
	}
	
	
	/**
	 * Singleton design pattern
	 * if no instance is used, create new instance, otherwise return known instance.
	 *
	 * @return single instance of Manager
	 */
	public static Manager getInstance()
	{
		if (instance == null) 
		{
			instance = new Manager();
		}
		return instance;
	}

	/**
	 * Gets the all printings.
	 *
	 * @return the all printings
	 */
	public ArrayList<Printing> getAllPrintings() {
		return printList;
	}
	

	/**
	 * Gets the printing by id.
	 *
	 * @param magazineID the magazine id
	 * @return the printing by id
	 */
	public Printing getPrintingById(int magazineID) {
		Iterator<Printing> actual = printList.iterator();
		while (actual.hasNext())
		{
			Printing current = actual.next();
			if (current.getId() == magazineID) return current;
		}
		return null;
	}
	
	public void safe(boolean how) throws FileNotFoundException, IOException{
		/* if how is false, we safe the objects in a simple file, else we safe it to an xml file */
		if(!how){
			storageDataAccess.serialize_to_disk(printList);
		}
		else
		{
			storageDataAccess.serialize_to_xml(printList);
		}				
	}
	public void load(boolean from) throws FileNotFoundException, ClassNotFoundException, IOException{
		if (!from){
			storageDataAccess.deserialize_from_disk();
		}
		else
		{
			storageDataAccess.deserialize_from_xml();
		}
	}
	public ArrayList<Printing> sortID(){
		Collection<Printing> collection = printList;
		List<Printing> list = new ArrayList<Printing>(collection);
		Collections.sort(list, new IDComparator());
		printList = (ArrayList<Printing>)list;
		return (ArrayList<Printing>) printList;
	}

	public ArrayList<Printing> sortTitle() {
		Collection<Printing> collection = printList;
		List<Printing> list = new ArrayList<Printing>(collection);
		Collections.sort(list, new TitleComparator());
		printList = (ArrayList<Printing>)list;
		return (ArrayList<Printing>) printList;
	}

	public ArrayList<Printing> sortType() {
		Collection<Printing> collection = printList;
		List<Printing> list = new ArrayList<Printing>(collection);
		Collections.sort(list, new TypeComparator());
		printList = (ArrayList<Printing>)list;
		return (ArrayList<Printing>) printList;
	}

	public ArrayList<Printing> sortTitle_was() {
		Printing[] printArray =  printList.toArray(new Printing[printList.size()]);
		Arrays.sort(printArray);
		printList.removeAll(printList);
		
		Collections.addAll(printList, printArray);
		
		return (ArrayList<Printing>)printList;
	}
}
