package applicationLogicAccess;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import applicationLogic.BadPrintingException;
import applicationLogic.InvalidInputException;
import applicationLogic.InvalidMagazineID;
import applicationLogic.Printing;
import applicationLogic.Manager;
import applicationLogic.Book;
import applicationLogic.Article;
import applicationLogic.Magazine;

/**
 * The Class Access.
 */
public class Access 
{
	
	/**
	 * Insert printing.
	 *
	 * @param toAdd the to add
	 * @return the int
	 */
	public static int insertPrinting(Printing toAdd) {
		return Manager.getInstance().addPrinting(toAdd);
	}
	
	/**
	 * Insert book.
	 *
	 * @param title the title
	 * @param author the author
	 * @param dateOfPublication the date of publication
	 * @param publisher the publisher
	 * @param isbn the isbn
	 * @return the int
	 */
	public static int insertBook(String title, String author, GregorianCalendar dateOfPublication, String publisher, String isbn) throws InvalidInputException
	{
		if (title.equals(title) && author.equals(author) && dateOfPublication.equals(dateOfPublication) && publisher.equals(publisher) && isbn.equals(isbn)){
			Book toAdd = new Book(title, author, dateOfPublication, publisher, isbn);
			return insertPrinting(toAdd);
		}
		else
		{
			throw new InvalidInputException();
		}
	}
	
	/**
	 * Insert magazine.
	 *
	 * @param title the title
	 * @param year the year
	 * @param edition the edition
	 * @param publisher the publisher
	 * @return the int
	 */
	public static int insertMagazine(String title, int year, int edition, String publisher) throws InvalidInputException
	{
		if (title.equals(title) && year == (int)year && edition == (int)edition && publisher.equals(publisher)){
			Magazine toAdd = new Magazine(title, year, edition, publisher);
			return insertPrinting(toAdd);
		}
		else
		{
			throw new InvalidInputException();
		}
	}
	
	/**
	 * Insert article.
	 *
	 * @param title the title
	 * @param year the year
	 * @param edition the edition
	 * @param author the author
	 * @param magazineID the magazine id
	 * @param page the page
	 * @return the int
	 * @throws InvalidMagazineID 
	 */
	public static int insertArticle(String title, int year, int edition, String author, int magazineID, int page) throws InvalidInputException, InvalidMagazineID
	{
		if (title.equals(title) && year == (int)year && edition == (int)edition && author.equals(author) && magazineID == (int)magazineID && page == (int)page){
			Printing magazine = Manager.getInstance().getPrintingById(magazineID);
			if ((magazine == null) || (!(magazine instanceof Magazine))) throw new InvalidMagazineID(magazineID); //error handling
			
			Article toAdd = new Article(title, year, edition, author, (Magazine)magazine, page);
			return insertPrinting(toAdd);
		}
		else
		{
			throw new InvalidInputException();
		}
	}
	
	/**
	 * Delete printing.
	 *
	 * @param id the id
	 * @return true, if successful
	 * @throws BadPrintingException 
	 */
	public static boolean deletePrinting(int id) throws BadPrintingException {
		return Manager.getInstance().deletePrinting(id);
	}
	
	/**
	 * Filter printings.
	 *
	 * @param type the type
	 * @return the array list
	 */
	public static ArrayList<Printing> filterPrintings(String type) throws InvalidInputException {
		if (type.equals(type)){
			return Manager.getInstance().filterPrintings(type);
		}
		else
		{
			throw new InvalidInputException();
		}
	}
	
	/**
	 * Gets the printings by name.
	 *
	 * @param name the name
	 * @return the printings by name
	 */
	public static ArrayList<Printing> getPrintingsByName(String name) throws InvalidInputException {
		if (name.equals(name)){
			return Manager.getInstance().searchPrintings(name);
		}
		else
		{
			throw new InvalidInputException();
		}
	}
	
	/**
	 * Gets the all printings.
	 *
	 * @return the all printings
	 */
	public static ArrayList<Printing> getAllPrintings(){
		return Manager.getInstance().getAllPrintings();
	}
	
	public static void safe(boolean how) throws FileNotFoundException, IOException{
		Manager.getInstance().safe(how);
	}
	public static void load(boolean from) throws FileNotFoundException, IOException, ClassNotFoundException{
		Manager.getInstance().load(from);
	}
	public static ArrayList<Printing> sortID(){
		return Manager.getInstance().sortID();
	}
	public static ArrayList<Printing> sortTitle(){
		return Manager.getInstance().sortTitle();
	}
	public static ArrayList<Printing> sortType(){
		return Manager.getInstance().sortType();
	}

	public static ArrayList<Printing> sortTitle_was() {
		return Manager.getInstance().sortTitle_was();
	}
}
